@@ -3,6 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2004,2005,2008 Olly Betts
+ * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,14 +23,12 @@
 
 #include <config.h>
 
-#include <limits.h>
+#include "flint_check.h"
 
-#include <iostream>
+#include <climits>
 
 using namespace std;
 
-#include "flint_check.h"
-
 #define REVISION(b)      static_cast<unsigned int>(getint4(b, 0))
 #define GET_LEVEL(b)     getint1(b, 4)
 #define MAX_FREE(b)      getint2(b, 5)
@@ -205,9 +204,10 @@ BtreeCheck::block_check(Cursor_ * C_, int j, int opts)
 }
 
 void
-BtreeCheck::check(const string & name, int opts, ostream &out)
+BtreeCheck::check(const char * tablename, const string & path, int opts,
+		  ostream &out)
 {
-    BtreeCheck B(name, false, out);
+    BtreeCheck B(tablename, path, false, out);
     B.open(); // throws exception if open fails
     Cursor_ * C = B.C;
 
