@@ -101,12 +101,12 @@ static void do_write(int fd, const char * p, size_t n)
 static off_t
 truncated_copy(const string & srcpath, const string & destpath, off_t tocopy)
 {
-    int fdin = open(srcpath.c_str(), O_RDONLY);
+    int fdin = open(srcpath.c_str(), O_RDONLY | O_BINARY);
     if (fdin == -1) {
 	FAIL_TEST("Open failed (when opening '" + srcpath + "')");
     }
 
-    int fdout = open(destpath.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
+    int fdout = open(destpath.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
     if (fdout == -1) {
 	FAIL_TEST("Open failed (when creating '" + destpath + "')");
     }
@@ -140,16 +140,19 @@ get_changeset(const string & changesetpath,
 	      Xapian::DatabaseReplica & replica,
 	      int expected_changesets,
 	      int expected_fullcopies,
-	      bool expected_changed)
+	      bool expected_changed,
+	      bool full_copy = false)
 {
-    int fd = open(changesetpath.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
+    int fd = open(changesetpath.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
     if (fd == -1) {
 	FAIL_TEST("Open failed (when creating a new changeset file at '"
 		  + changesetpath + "')");
     }
     fdcloser fdc(fd);
     Xapian::ReplicationInfo info1;
-    master.write_changesets_to_fd(fd, replica.get_revision_info(), &info1);
+    master.write_changesets_to_fd(fd,
+				  full_copy ? "" : replica.get_revision_info(),
+				  &info1);
 
     TEST_EQUAL(info1.changeset_count, expected_changesets);
     TEST_EQUAL(info1.fullcopy_count, expected_fullcopies);
@@ -163,7 +166,7 @@ apply_changeset(const string & changesetpath,
 		int expected_fullcopies,
 		bool expected_changed)
 {
-    int fd = open(changesetpath.c_str(), O_RDONLY);
+    int fd = open(changesetpath.c_str(), O_RDONLY | O_BINARY);
     if (fd == -1) {
 	FAIL_TEST("Open failed (when reading changeset file at '"
 		  + changesetpath + "')");
@@ -199,13 +202,15 @@ replicate(Xapian::DatabaseMaster & master,
 	  const string & tempdir,
 	  int expected_changesets,
 	  int expected_fullcopies,
-	  bool expected_changed)
+	  bool expected_changed,
+	  bool full_copy = false)
 {
     string changesetpath = tempdir + "/changeset";
     get_changeset(changesetpath, master, replica,
 		  expected_changesets,
 		  expected_fullcopies,
-		  expected_changed);
+		  expected_changed,
+		  full_copy);
     return apply_changeset(changesetpath, replica,
 			   expected_changesets,
 			   expected_fullcopies,
@@ -288,7 +293,7 @@ DEFINE_TESTCASE(replicate1, replicas) {
 
     // Apply the replication - we don't have changesets stored, so this should
     // just do a database copy, and return a count of 1.
-    int count = replicate(master, replica, tempdir, 0, 1, 1);
+    int count = replicate(master, replica, tempdir, 0, 1, true);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -297,7 +302,7 @@ DEFINE_TESTCASE(replicate1, replicas) {
 
     // Repeating the replication should return a count of 1, since no further
     // changes should need to be applied.
-    count = replicate(master, replica, tempdir, 0, 0, 0);
+    count = replicate(master, replica, tempdir, 0, 0, false);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -308,7 +313,7 @@ DEFINE_TESTCASE(replicate1, replicas) {
     // to occur, whether it was needed or not.  Fixed in revision #10117.
     replica.close();
     replica = Xapian::DatabaseReplica(replicapath);
-    count = replicate(master, replica, tempdir, 0, 0, 0);
+    count = replicate(master, replica, tempdir, 0, 0, false);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -320,7 +325,7 @@ DEFINE_TESTCASE(replicate1, replicas) {
     orig.add_document(doc1);
     orig.commit();
 
-    count = replicate(master, replica, tempdir, 2, 0, 1);
+    count = replicate(master, replica, tempdir, 2, 0, true);
     TEST_EQUAL(count, 3);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -366,11 +371,11 @@ DEFINE_TESTCASE(replicate2, replicas) {
 
     // Apply the replication - we don't have changesets stored, so this should
     // just do a database copy, and return a count of 1.
-    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, 1), 1);
+    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, true), 1);
     check_equal_dbs(masterpath, replicapath);
 
     // Replicate from the replica.
-    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, 1), 1);
+    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, true), 1);
     check_equal_dbs(masterpath, replica2path);
 
     orig.add_document(doc1);
@@ -379,7 +384,7 @@ DEFINE_TESTCASE(replicate2, replicas) {
     orig.commit();
 
     // Replicate from the replica - should have no changes.
-    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 0, 0), 1);
+    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 0, false), 1);
     check_equal_dbs(replicapath, replica2path);
 
     // Replicate, and replicate from the replica - should have 2 changes.
@@ -396,9 +401,9 @@ DEFINE_TESTCASE(replicate2, replicas) {
     orig.commit();
 
     // Replication should do a full copy.
-    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, 1), 1);
+    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, true), 1);
     check_equal_dbs(masterpath, replicapath);
-    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, 1), 1);
+    TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, true), 1);
     check_equal_dbs(masterpath, replica2path);
 
     // Start writing changesets, but only keep 1 in history, and make a
@@ -423,9 +428,9 @@ DEFINE_TESTCASE(replicate2, replicas) {
     // is missing.
 
     //FIXME - the following tests are commented out because the backends don't currently tidy up old changesets correctly.
-    //TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, 1), 1);
+    //TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, true), 1);
     //check_equal_dbs(masterpath, replicapath);
-    //TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, 1), 1);
+    //TEST_EQUAL(replicate(master2, replica2, tempdir, 0, 1, true), 1);
     //check_equal_dbs(masterpath, replica2path);
 
     // Need to close the replicas before we remove the temporary directory on
@@ -464,7 +469,7 @@ replicate_with_brokenness(Xapian::DatabaseMaster & master,
 				       expected_changed));
 	if (len < 30 || len >= filesize - 10) {
 	    // For lengths near the beginning and end, increment size by 1
-	    len += 1;
+	    ++len;
 	} else {
 	    // Don't bother incrementing by small amounts in the middle of
 	    // the changeset.
@@ -474,7 +479,6 @@ replicate_with_brokenness(Xapian::DatabaseMaster & master,
 	    }
 	}
     }
-    return;
 }
 
 // Test changesets which are truncated (and therefore invalid).
@@ -499,14 +503,14 @@ DEFINE_TESTCASE(replicate3, replicas) {
     orig.add_document(doc1);
     orig.commit();
 
-    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, 1), 1);
+    TEST_EQUAL(replicate(master, replica, tempdir, 0, 1, true), 1);
     check_equal_dbs(masterpath, replicapath);
 
     // Make a changeset.
     orig.add_document(doc1);
     orig.commit();
 
-    replicate_with_brokenness(master, replica, tempdir, 1, 0, 1);
+    replicate_with_brokenness(master, replica, tempdir, 1, 0, true);
     // Although it throws an error, the final replication in
     // replicate_with_brokenness() updates the database, since it's just the
     // end-of-replication message which is missing its body.
@@ -516,7 +520,7 @@ DEFINE_TESTCASE(replicate3, replicas) {
     // next replication.
     orig.add_document(doc1);
     orig.commit();
-    TEST_EQUAL(replicate(master, replica, tempdir, 1, 0, 1), 2);
+    TEST_EQUAL(replicate(master, replica, tempdir, 1, 0, true), 2);
 
     // Need to close the replicas before we remove the temporary directory on
     // Windows.
@@ -548,7 +552,7 @@ DEFINE_TESTCASE(replicate4, replicas) {
 
     // Apply the replication - we don't have changesets stored, so this should
     // just do a database copy, and return a count of 1.
-    int count = replicate(master, replica, tempdir, 0, 1, 1);
+    int count = replicate(master, replica, tempdir, 0, 1, true);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -561,7 +565,7 @@ DEFINE_TESTCASE(replicate4, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -577,7 +581,7 @@ DEFINE_TESTCASE(replicate4, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -597,7 +601,7 @@ DEFINE_TESTCASE(replicate4, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 0, 1, 1);
+    count = replicate(master, replica, tempdir, 0, 1, true);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -641,7 +645,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
 
     // Apply the replication - we don't have changesets stored, so this should
     // just do a database copy, and return a count of 1.
-    int count = replicate(master, replica, tempdir, 0, 1, 1);
+    int count = replicate(master, replica, tempdir, 0, 1, true);
     TEST_EQUAL(count, 1);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -654,7 +658,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -670,7 +674,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -686,7 +690,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -710,7 +714,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -726,7 +730,7 @@ DEFINE_TESTCASE(replicate5, replicas) {
     orig.commit();
 
     // Replicate, and check that we have the positional information.
-    count = replicate(master, replica, tempdir, 1, 0, 1);
+    count = replicate(master, replica, tempdir, 1, 0, true);
     TEST_EQUAL(count, 2);
     {
 	Xapian::Database dbcopy(replicapath);
@@ -745,3 +749,66 @@ DEFINE_TESTCASE(replicate5, replicas) {
     rmtmpdir(tempdir);
     return true;
 }
+
+/// Test --full-copy option.
+DEFINE_TESTCASE(replicate6, replicas) {
+    UNSET_MAX_CHANGESETS_AFTERWARDS;
+    string tempdir = ".replicatmp";
+    mktmpdir(tempdir);
+    string masterpath = get_named_writable_database_path("master");
+
+    set_max_changesets(10);
+
+    Xapian::WritableDatabase orig(get_named_writable_database("master"));
+    Xapian::DatabaseMaster master(masterpath);
+    string replicapath = tempdir + "/replica";
+    Xapian::DatabaseReplica replica(replicapath);
+
+    // Add a document to the original database.
+    Xapian::Document doc1;
+    doc1.set_data(string("doc1"));
+    doc1.add_posting("doc", 1);
+    doc1.add_posting("one", 1);
+    orig.add_document(doc1);
+    orig.commit();
+
+    rm_rf(masterpath + "1");
+    cp_R(masterpath, masterpath + "1");
+
+    orig.add_document(doc1);
+    orig.commit();
+
+    // Apply the replication - we don't have changesets stored, so this should
+    // just do a database copy, and return a count of 1.
+    int count = replicate(master, replica, tempdir, 0, 1, true);
+    TEST_EQUAL(count, 1);
+    {
+	Xapian::Database dbcopy(replicapath);
+	TEST_EQUAL(orig.get_uuid(), dbcopy.get_uuid());
+    }
+
+    Xapian::DatabaseMaster master1(masterpath + "1");
+
+    // Try to replicate an older version of the master.
+    count = replicate(master1, replica, tempdir, 0, 0, false);
+    TEST_EQUAL(count, 1);
+
+    // Force a full copy.
+    count = replicate(master1, replica, tempdir, 0, 1, true, true);
+    TEST_EQUAL(count, 1);
+
+    // Test we can still replicate.
+    orig.add_document(doc1);
+    orig.commit();
+
+    count = replicate(master, replica, tempdir, 2, 0, true);
+    TEST_EQUAL(count, 3);
+
+    check_equal_dbs(masterpath, replicapath);
+
+    // Need to close the replica before we remove the temporary directory on
+    // Windows.
+    replica.close();
+    rmtmpdir(tempdir);
+    return true;
+}
